#!/usr/bin/python
'''ZetaBoards to phpBB 2 Converter by nneonneo
Version 0.1.6
Converts Categories, Forums, Members, Topics and Posts.
Mar 2 2008
Updated Jan 01 2011'''
import os,re,sys,random
from time import strptime,strftime,mktime,time,localtime
try:
    from settings import *
    COOKIEDATA=get_cookie()
except Exception, detail:
    print "Error: Settings file has some error."
    print "Details of the error:"
    sys.excepthook(*sys.exc_info())
    raw_input("Press <enter> to quit.")
    sys.exit(1)
sys.path.append('..')
from common import *

# Don't edit below here [unless you know exactly what you are doing]
members_max_group = 500
topics_max_group = 500
posts_max_group = 200
beg_time=time()

boards=[]

# topics by board [board maps to topic list]
topicslist={}

# members [maps member names to ids]
memberslist={'guest hack':(-1, "Guest")} # the space ensures that "real" users will not have this ID

# map old topicids to new ones; format '%i/%s'%(topicid,board):newid
topicmap={}

# category detection
re_cats=re.compile('''(?sm)<div id="cat-(\\d+)" class="category".*?<a href='[^']+\\1/'>([^<]*)</a>(.+?)^		</div>''')

# forum detection
re_forums=re.compile('''(?sm)<tr id="forum-(\\d+)" class="forum">.+?<a href="[^"]+\\1/">(.+?)</a></strong>\\s*(?:<div>(.*?)(?:<br />Moderated By: .+?)?</div>$)?''')

# subforum detection
re_subforums=re_forums

# member page detection, page detection
re_pages=re_member_pages=re.compile('''(?sm)<ul class='cat-pages'><li class="cat-pageshead">Pages:</li>.+?>(\\d+)</a></li>\\s*</ul>''')

# member detection
member_search=re.compile('''<td><a href='[^']+/profile/(\\d+)/' class='member'>(?:<strong>)?(?:<span class="member">)?(.+?)(?:</span>)?(?:</strong>)?</a></td>''')

# thread ID detection
# 1:pin  2:lock  3:id  4:views
thread_search=re.compile('''(?s)<tr class="row\\d*( pin)?(?: newreplies)?">.+?<td class="c_cat-mark"><img[^>]*src=['"]([^'"]*)['"][^>]*></td>.*?\
<td class="c_cat-title">.+?<a href="[^"]+/(\\d+)/" title="Start Date.+?<td class="c_cat-views">\\s*([\\d,]+)\\s*</td>''')

# the massive grep list of doom!
# the grep list works by changing certain things to delimited, structured data. The delimiters are "<^>" at the start and "</^>" at the end of a record, and the fields are separated with "<|>".
# first one: detection of the exact forum we're in
repstrings=[("""<forum_id=(\\d+)>""","""<^>FORUM<|>\\1</^>"""),
# detect the title of the topic
("""<head><title>([^<]+)</title>""","""<^>TITLE<|>\\1</^>"""),
# re hack: replace "unreg" with a registered user, id 0. RE doesn't work right :(
("""<td class="c_username">(\\s+)([^<\\s][^<]*)<""","""<td class="c_username">\\1<a href='http://s1.zetaboards.com/mojibake/profile/guest hack/' class='member'>\\2</a><"""),
# !!! the big one: get the entire post
("""(?sm)<tr id="post-(\\d+)">.+?<td class="c_username">\\s+<a href='[^']+/profile/([^/]+)/' class='member'>(.+?)</a>.+?\
<td class="c_postinfo">\\s+<span class="left">\\s*(.+?)\\s*</span>.+?<td class="c_post">\\s*(.+?)\\s*\t\t\t\t\t</td>""",
 """<^>POST<|>\\1<|>\\2<|>\\3<|>\\4<|>\\5</^>"""),
# clarify the edit-by information; this gets added as optional extra fields
("""\\s*<div class="editby">Edited by \
(?:<strong>)?(?:<a href='[^']+/profile/[^/]+/'>)?(.+?)(?:</a>)?(?:</strong>)?, (.+?)\\..*?</div>\\s*</\\^>""",
 """<|>EDIT<|>\\1<|>\\2</^>"""),
# clean up attachments
("""\\s*<dl class="c_post-attachment">\\s*<dt>.+?</dt>\\s*""",'<br /><br /><b>Attachments:</b><br /><ul>'),
("""</dd>\\s+</dl>""","</dd></ul>"),
("""<dd><a href="([^"]+)"><img[^>]*>(.+?)</a></dd>\\s*""","""<li>[url=\\1]\\2[/url]</li>"""),
# add a "<^>" tag to the end of the file - acts like the start of a tag
("""\\Z""","""<^>"""),
# and a "</^>" tag to the beginning - like the end of a tag
("""\\A""","""</^>"""),
# delete everything between the tags (blank space) leaving only the tags
("""</\\^>[\\w\\W]*?<\\^>""","""</^><^>"""),
# kill duplicate data from subsequent pages
("""POST<\\|>(.+)\\nTITLE<\\|>.+""","""POST<|>\\1"""),
# convert image links
("""<img src="([^"]+)" alt="Posted Image" />""","""[img:<UID>]\\1[/img:<UID>]"""),
# convert emotes
("""<img src="[^"]+" alt="([^"]+)" />""","""\\1"""),
# flash video
("""<object type='application/x-shockwave-flash' width='(\\d+)' height='(\\d+)' data='([^']+)'>
\\s*<param name='AllowScriptAccess' value='never' />
\\s*<param name='wmode' value='transparent' />
\\s*<param name='movie' value='\\3' /><param name='play' value='true' />
\\s*<param name='loop' value='true' /><param name='quality' value='high' /></object>""","""[url]\\3[/url]"""),
# destroy any blank lines
("""^\\n""",""""""),
# and remove the blank line at the end.
("""\\n\\Z""",""""""),
# code tags
("""<blockquote><dl><dt>Code: ?</dt><dd>&nbsp;</dd></dl><code>(.+?)</code></blockquote>""","""[code:1:<UID>]\\1[/code:1:<UID>]"""),
# html tags
("""<blockquote><dl><dt>Code: HTML</dt><dd>&nbsp;</dd></dl><code>(.+?)</code></blockquote>""","""\\1"""),
# quotes: no author
("""<blockquote><dl><dt>Quote:</dt><dd>&nbsp;</dd></dl><div>""", "[quote:<UID>]"),
# quotes: anything else
("""<blockquote><dl><dt>([^<]*)</dt><dd>([^<]*)</dd></dl><div>""", '[quote:<UID>="\\1"]'),
# quotes: closing
("""</div></blockquote>""", "[/quote:<UID>]"),
# b,i,u tags
("<(/?[biu])>","[\\1:<UID>]"),
# strong, em tags
("<(/?)strong>","[\\1b:<UID>]"),
("<(/?)em>","[\\1i:<UID>]"),
# emails
("""<a href="mailto:([^"]+)" target="_blank">(.+?)</a>""","""[email]\\1[/email]"""),
# convert URL links
("""<a href="([^"]+)" target="_blank" rel="nofollow">(.+?)</a>""","""[url=\\1]\\2[/url]"""),
# [me]
("""<a href='(.+?)' class='member'>(.+?)</a>""","""\\2"""),
# list tags (i and I not supported)
('<li style="display:none">.*?</li>',''), # null list item
('<ol>',"[list=1:<UID>]"),
('<ol style="list-style-type:decimal">',"[list=1:<UID>]"),
('<ol style="list-style-type:lower-alpha">',"[list=a:<UID>]"),
('<ol style="list-style-type:upper-alpha">',"[list=A:<UID>]"),
("<ul>","[list:<UID>]"), # unordered
("</ol>","[/list:o:<UID>]"), # closing tag
("</ul>","[/list:u:<UID>]"), # closing tag
("<li/>","[*:<UID>]"),
("<li>","[*:<UID>]"), # list item
("</li>","\n"), # no end tag needed, but we need to force a linebreak to end [*]
# convert newlines
("<br />", "\n")
]
re_repstrings=[]
for a,b in repstrings: re_repstrings+=[(re.compile(a),b)] # compile all search patterns for increased efficiency; replace patterns aren't normal grep so they can't be compiled

def parsetime(tm,fmt="%b %d %Y, %I:%M:%S %p"): # try a bunch of formats, including defaults (less accurate)
    # try to handle relative times, but you really should turn them off
    if tm=='One minute ago':
        return time()-60
    elif tm.endswith(" minutes ago"):
        mins=tm.split(' ',1)[0]
        return time()-int(mins)*60
    tm=tm.replace("Today, ",strftime('%b %d %Y, ')).\
          replace("Yesterday, ",strftime('%b %d %Y, ',localtime(time()-24*3600)))
    for f in [fmt, '%b %d %Y, %I:%M %p', '%b %d %Y', '%b %d, %Y']:
        try: return mktime(strptime(tm,f))
        except Exception: pass
    raise ValueError("Couldn't parse the date %r."%tm)

def parsespans(dat):
    def ishex(s):
        for i in s:
            if i.upper() not in '1234567890ABCDEF': return False
        return True
    '''<span style='font-family:Courier'><span style='font-size:14pt;line-height:100%'><span style='color:#468212'>PWNAGE</span></span></span>'''
    '''[font=Courier][SIZE=7][color=#468212]PWNAGE[/SIZE][/color][/font]'''
    '''[pre][size=14pt][color=#468212]PWNAGE[/size][/color][/pre]'''
    sb="<span"
    se='</span>'
    begin_off=dat.find(sb)
    if begin_off==-1: return dat # no spans!
    out=dat[:begin_off] # data right up to the span
    rest=dat[begin_off+len(sb):] # the remainder, after the sb block
    lwrrest=rest.lower()
    if lwrrest.startswith(' style="font-family:'):
        type="font"
        starttag='[font='+rest[len(' style="font-family:'):].split('"')[0]+']'
        endtag='[/font]'
    elif lwrrest.startswith(' style="font-weight:bold;color:'):
        type="color"
        color=rest[len(' style="font-weight:bold;color:'):].split('"')[0]
        starttag='[b:<UID>][color='+color+':<UID>]'
        endtag='[/color:<UID>][/b:<UID>]'
    elif lwrrest.startswith(' style="color:'):
        type="color"
        color=rest[len(' style="color:'):].split('"')[0]
        if color.startswith('#'):
            color=color[1:]
            if len(color) == 3 and ishex(color):
                # short form colors; must expand to fit 6 chars
                color='#' + (color[0]*2 + color[1]*2 + color[2]*2).upper()
            elif len(color) == 6 and ishex(color):
                color='#' + color.upper() #hexcolors
        else: color=color.lower()
        starttag='[color='+color+':<UID>]'
        endtag='[/color:<UID>]'
    else:
        starttag=endtag=type=''
    spandata=starttag
    se_off=rest.find(">") # nearest span ending
    if se_off==-1:
        print "Warning: Invalid <span> found!!"
        return dat
    rest=rest[se_off+1:] # what's after the > ?
    text=rest
    nestlevel=1 # nesting level: we're inside one span
    while nestlevel:
        se_off=rest.find(se) # where's the </span>
        sb_off=rest.find(sb) # or another <span
        if se_off==-1: 
            print "Warning: No </span> found!!"
            return dat
        if sb_off==-1 or se_off<sb_off: # no new beginning (sb) before the se? good news!
            nestlevel-=1
            rest=rest[se_off+len(se):]
            continue
        elif sb_off!=-1 and sb_off<se_off: # a new beginning? darn. process it. new beginning must be before the end.
            rest=rest[sb_off+len(sb):] 
            nestlevel+=1 # increase nesting level, we won't actually process it, this just helps us find the exact block of this span
            continue
    spandata+=parsespans(text[:-(len(rest)+len(se))])+endtag # gotcha! the end of the span. Now, process the innards for nested spans.
    spandata+=parsespans(rest) # parse the rest
    return out+spandata

def dl_cvt_subforum(fid, level, parent, name, desc):
    ''' recursively download and convert subforums '''
    global forumnumber, catnumber, out
    selfnumber=BOARDS[fid]=forumnumber
    out.write('INSERT INTO `'+table_prefix+'forums` (forum_id, cat_id, forum_order, forum_name, forum_desc) VALUES (%s, %s, %s, \'%s\', \'%s\');\n'%((forumnumber, catnumber, forumnumber*10)+tuple(escape_vars([name, desc]))))
    forumnumber+=1
    statusline="Forum %s ... "%fid
    progressline=statusline+"Downloading - "
    forumdata = download_page(URL+"forum/%s/"%fid, progressline, COOKIEDATA)
    subforums = re.findall(re_subforums, forumdata)
    for sfid, sname, sdesc in subforums:
        dl_cvt_subforum(sfid, level+1, selfnumber, sname, sdesc)

postid=1
BOARDS={}
topicdata,postdata={},[]

print "Downloading Forum Structure..."
out=open("categories_forums_phpBB.sql","w")
out.write('TRUNCATE TABLE `'+table_prefix+'forums`; TRUNCATE TABLE `'+table_prefix+'categories`;\n')
try:
    statusline="Categories ... "
    progressline=statusline+"Downloading - "
    pagedata = download_page(URL+'index/', progressline, COOKIEDATA)
    cats=re.findall(re_cats,pagedata)
    if not cats:
        print
        print "Failed to get any categories :("
        print "Please notify nneonneo."
        raw_input("Press <enter> to exit.")
        sys.exit(1)
    catnumber = 1
    forumnumber = 1
    for catid, name, data in cats:
        out.write('INSERT INTO `'+table_prefix+'categories` (cat_id, cat_order, cat_title) VALUES (%s, %s, \'%s\');\n'%(catnumber, catnumber*10, escape_vars([name])[0]))
        for fid, name, desc in re.findall(re_forums, data):
            dl_cvt_subforum(fid, 0, 0, name, desc)
        catnumber += 1
    out.close()
    printline(statusline+"Completed!")
    print
except Exception:
    do_except("Categories and Forums", statusline)

print "Downloading Memberlist..."
out=open('members_phpBB.sql','w')
out.write('TRUNCATE TABLE `'+table_prefix+'users`;'+'TRUNCATE TABLE `'+table_prefix+'user_group`;'+'TRUNCATE TABLE `'+table_prefix+'groups`;INSERT INTO '+table_prefix+'''users (user_id, user_active, username, user_password, user_session_time, user_session_page, user_lastvisit, user_regdate, user_level, user_posts, user_timezone, user_style, user_lang, user_dateformat, user_new_privmsg, user_unread_privmsg, user_last_privmsg, user_login_tries, user_last_login_try, user_emailtime, user_viewemail, user_attachsig, user_allowhtml, user_allowbbcode, user_allowsmile, user_allowavatar, user_allow_pm, user_allow_viewonline, user_notify, user_notify_pm, user_popup_pm, user_rank, user_avatar, user_avatar_type, user_email, user_icq, user_website, user_from, user_sig, user_sig_bbcode_uid, user_aim, user_yim, user_msnm, user_occ, user_interests, user_actkey, user_newpasswd) VALUES('-1', '0', 'Anonymous', '', '0', '0', '0', '0', '0', '0', '0.00', NULL, 'english', '', '0', '0', '0', '0', '0', NULL, '0', '0', '1', '1', '1', '1', '0', '1', '0', '1', '0', NULL, '', '0', '', '', '', '', '', NULL, '', '', '', '', '', '', '');INSERT INTO '''+table_prefix+'''groups (group_id, group_type, group_name, group_description, group_moderator, group_single_user) VALUES('1', '1', 'Anonymous', 'Personal User', '0', '1');INSERT INTO '''+table_prefix+'''user_group (group_id, user_id, user_pending) VALUES('1', '-1', '0');''')
PM_data=open('pm_passwords.txt','w')
memberdata=[]
try:
    statusline="Members ... "
    progressline=statusline+"Downloading - "
    pagedata = download_page(URL+"members/?search_type=start&name=&group=0&sort=join_unix&order=a", progressline, COOKIEDATA)
    pages=re.findall(re_member_pages,pagedata)
    if pages:
        np=int(pages[0])
        for offset in range(2,np+1):
            progressline=statusline+"Downloading Page %i of %i - "%(offset,np)
            pagedata += download_page(URL+"members/"+str(offset)+"/?search_type=start&name=&group=0&sort=join_unix&order=a", progressline, COOKIEDATA)
    printline(statusline+"Finding Members")
    for memberid, name in re.findall(member_search,pagedata):
        try:
            random.seed(RANDOMNUMBER+hash(memberid)) # creates static member passwords: they will never change.
            statusline="Member %s (ID %s) ... "%(name, memberid)
            progressline=statusline+"Downloading - "
            userdata=download_page(URL+"profile/%s/"%memberid, progressline, COOKIEDATA)
            p=re.findall("""<tr>\\s+<td class="c_desc">([^<]+)</td>\\s*<td>(.+?)</td>\\s+</tr>""", userdata)
            p+=re.findall("""<dt>([^<]+)</dt>\\s*<dd>(.+?)</dd>""",userdata)
            if not p: # oops, no profile lines found
                printline(statusline+"Warning: Member doesn't exist!")
                print
                continue
            """
            Indices of "dat" mapped to phpBB phpbb_users columns
            0:user_id
            1:username
            2:[realName]
            3:user_email
            4:[websiteTitle]
            5:user_website
            6:[birthdate]
            7:[gender]
            8:user_from
            9:user_posts
            10:[karmaGood] <-- warning, could be negative; I wonder how that's handled?
            11:user_regdate
            12:user_icq
            13:user_aim
            14:user_yim
            15:user_msnm
            16:user_lastvisit
            17:user_password
            18:user_avatar
            19:user_level
            20:user_interests
            """
            name = re.findall("<head><title>(.+?)</title>",userdata)[0]
            dat=[memberid,name,name,'','','','0001-01-01',0,'',0,0,0,'','','','',0,'','',0,'']
            dat[18]=(re.findall("<img src='([^']+)' border='0' alt='Member Avatar' class='avatar' style='max-width: 100px; max-height: 100px;' />",userdata) or [''])[0]

            data=download_page(ZBURL+"admin/?menu=mem&c=1&mid=%s"%memberid,statusline+"Retrieving e-mail address...",COOKIEDATA)
            found=re.findall("""<input[^>]*name='email'[^>]*value='([^']*)'""",data)
            if not found:
                print
                print "\tError: Couldn't retrieve e-mail address from admin panel."
            else:
                dat[3]=found[0]

            printline(statusline+"Processing")
            for t,i in p:
                try:
                    t=t.strip(':')
                    i=i.strip()
                    if t=='Name': pass # this name isn't used anywhere else on the forum, so it doesn't correspond to realName
                    elif t=='Website': dat[5],dat[4]=re.findall("""<a href='(.+?)' rel='nofollow' target='_blank'>(.+?)</a>""",i)[0]
                    elif t=='Birthday':
                        try:
                            dat[6]=strftime('%Y-%m-%d',strptime(i,'%B %d, %Y'))
                        except Exception:
                            dat[6]=strftime('0000-%m-%d',strptime(i,'%B %d'))
#                    elif t in ['Gender','Sex']: dat[7]={'Undisclosed':0,'Male':1,'Female':2}[i]
                    elif t=='Location': dat[8]=i
                    elif t=='Posts': dat[9]=int(i.replace(',',''))
                    elif t=='Group':
                        if groups.get(i,0)==1:
                            dat[19]=1
                    elif t=='Joined': dat[11]=int(parsetime(i))
                    elif t=='AOL IM Name': dat[13]=i
                    elif t=='Yahoo! Name': dat[14]=i
                    elif t=='MSN Name': dat[15]=i
                    elif t=='Last Activity':
                        dat[16]=int(parsetime(i))
                    elif t=='Average Posts Per Day': pass
                    elif t=='Member&#39;s Local Time': pass
                    elif t=='Online?': pass
                    elif t=='Email Address': pass
                    elif t=='Personal Messenger': pass
                    elif t=='Interests': dat[20]=i.replace('<br />','\n')
                    elif t=='Registered with IP Address': pass
                    elif t=='Member': pass
                    else:
                        print # blank line; prevent overwrite
                        print "\tUnknown field %s, data %s"%(t,i)
                except ValueError,detail:
                    print # blank line to prevent overwrite
                    print "\tError:",detail
            password=randpw()
            dat[17]=md5hex(password)
            PM_data.write("%s|%s|%s\n"%(name,memberid,password))
            printline(statusline+"Completed! User Password: %s"%password)
            memberdata+=[dat]
            print
        except KeyboardInterrupt: break # abort processing, write out data now
        except Exception:
            do_except('Member %s'%name, statusline)
    print # blank line for next part
except Exception:
    do_except('Members', statusline)
for i,dat in enumerate(memberdata):
    memberslist[dat[0]]=(i+1, dat[1])
    memberdata[i][0]=i+1
    dat[1]=dat[2] # set login name to real name
memberdata=[escape_vars(i) for i in memberdata]
for i in range(0,len(memberdata),members_max_group):
    out.write("INSERT INTO `"+table_prefix+"users` (user_id, username, user_email, user_website, user_from, user_posts, user_regdate, user_icq, user_aim, user_yim, user_msnm, user_lastvisit, user_password, user_avatar, user_level, user_occ, user_interests, user_sig, user_viewemail, user_style, user_attachsig, user_lang, user_actkey, user_newpasswd, user_avatar_type) VALUES ")
    out.write(',\n'.join(["(%i, '%s', '%s', '%s', '%s', %i, %i, '%s', '%s', '%s', '%s', %i, '%s', '%s', '%s', '', '%s', '', 0, NULL, 0, '', '', '', '2')"%(dat[0],dat[1],dat[3],dat[5],dat[8],dat[9],dat[11],dat[12],dat[13],dat[14],dat[15],dat[16],dat[17],dat[18],dat[19],dat[20]) for dat in memberdata[i:i+members_max_group]]))
    out.write(";\n")
    out.write("INSERT INTO `"+table_prefix+"groups` (group_id, group_type, group_name, group_description, group_moderator, group_single_user) VALUES ")
    out.write(',\n'.join(["('%i', '1', '%s', 'Personal User', '0', '1')"%(dat[0]+1,'Admin'*dat[19]) for dat in memberdata[i:i+members_max_group]]))
    out.write(";\n")
    out.write("INSERT INTO `"+table_prefix+"user_group` (group_id, user_id, user_pending) VALUES ")
    out.write(',\n'.join(["('%i', '%i', '0')"%(dat[0]+1,dat[0]) for dat in memberdata[i:i+members_max_group]]))
    out.write(";\n")
PM_data.close()
out.close()

print "Downloading Boards..."
for board in BOARDS:
    try:
        statusline="Board %s ... "%board
        progressline=statusline+"Downloading - "
        pagedata=download_page(URL+"forum/%s/1/?x=90&cutoff=100"%board, progressline, COOKIEDATA)
        pages=re.findall(re_pages,pagedata)
        if pages:
            np=int(pages[0])
            for offset in range(2,np+1):
                progressline=statusline+"Downloading Extra Page %i of %i - "%(offset-1,np-1)
                pagedata+=download_page(URL+"forum/%s/%i/?x=90&cutoff=100"%(board,offset), progressline, COOKIEDATA)
        printline(statusline+"Finding Topics")
        added_topics=re.findall(thread_search,pagedata)
        topicslist[board]=dict([(int(topic_id),('pin' in pinned,locked==lockimage,int(views.replace(',','')))) for pinned,locked,topic_id,views in added_topics])
        printline(statusline+"Completed: %i topic%s"%(len(added_topics),int(len(added_topics)!=1)*'s'))
        print # blank line for next board
    except Exception:
        do_except("Board ID %s"%board, statusline)
topictemp=[]
for board in topicslist:
    topictemp+=[(i,board) for i in topicslist[board]]
topictemp.sort(key=lambda x:x[0]) # sort by ID only
for i,(topicid,board) in enumerate(topictemp):
    topicmap['%i/%s'%(topicid,board)]=i+1

''' This section will go through the topic IDs found before and download each topic.
The data is then processed and distilled into posts via the multiple regex's above.'''
print "Downloading Topics and Posts..."
for board in topicslist:
    for topicid in topicslist[board]:
        try:
            viewcount=topicslist[board][topicid][2]
            statusline="Topic %i, Board %s ... "%(topicid,board)
            progressline=statusline+"Downloading - "
            pagedata=download_page(URL+"topic/%i/1/?x=90"%topicid, progressline, COOKIEDATA)
            pages=re.findall(re_pages,pagedata)
            if pages:
                np=int(pages[0])
                for offset in range(2,np+1):
                    progressline=statusline+"Downloading Page %i of %i - "%(offset,np) # 2 of 3, 3 of 3
                    pagedata+=download_page(URL+"topic/%i/%i/?x=90"%(topicid,offset), progressline, COOKIEDATA)
            processed_data='  <forum_id=%s>'%BOARDS[board]+pagedata
#           file("%i.txt"%topicid,"w").write(processed_data)
            cn=0
            for search,repl in re_repstrings:
                cn+=1
                printline(statusline+"Processing Step %i of %i"%(cn,len(re_repstrings)))
                processed_data=re.sub(search,repl,processed_data)
#               file("%i_%i.txt"%(topicid,cn),"w").write(processed_data)
            printline(statusline+"Converting Topic")
            forum,title=0,''
            posts=[]
            '''   0    1       2       3         4         5    6    7         8           '''
            '''<^>POST|Post ID|User ID|User Name|Date/Time|Text|EDIT|Edit Name|Edit Time</^>'''
            ''' 0    1  2      3    4        5        6    '''
            '''(time,id,userid,name,edittime,editname,data)'''
            for line in processed_data.split('</^><^>'):
                ln=line.strip()
                if not ln: continue
                ln=ln.split('<|>')
                if ln[0]=='FORUM':
                    forum=int(ln[1])
                    continue
                elif ln[0]=='TITLE':
                    title=ln[1]
                    continue
                elif ln[0]=='POST':
                    post=[0,0,-1,'',0,'','']
                    if len(ln)==9:
                        if ln[6]=='EDIT':
                            post[4]=parsetime(ln[8])
                            post[5]=ln[7]
                    post[0]=parsetime(ln[4].strip('(').strip(')')) # brackets? wtf?
                    post[1]=int(ln[1])
                    if ln[2] and ln[2] in memberslist:
                        post[2]=memberslist[ln[2]][0]
                    if post[2] > 0:
                        post[3]=memberslist[ln[2]][1]
                    else:
                        post[3]=re.sub(r'\[[^\]]*\]', '', parsespans(ln[3]))
                    post[6]=parsespans(ln[5].strip())
                    posts+=[post]
            if not posts:
                printline(statusline+"Error: Topic is empty!")
                print # a blank line, for the next topic
                continue
            posts.sort(key=lambda x:x[0]) # sort by date code only
            # This is for phpBB
            '''
            topic_id mediumint(8) UNSIGNED NOT NULL auto_increment,
            forum_id smallint(8) UNSIGNED DEFAULT '0' NOT NULL,
            topic_title char(60) NOT NULL,
            topic_poster mediumint(8) DEFAULT '0' NOT NULL,
            topic_time int(11) DEFAULT '0' NOT NULL,
            topic_views mediumint(8) UNSIGNED DEFAULT '0' NOT NULL,
            topic_replies mediumint(8) UNSIGNED DEFAULT '0' NOT NULL,
            topic_status tinyint(3) DEFAULT '0' NOT NULL,
            topic_vote tinyint(1) DEFAULT '0' NOT NULL,
            topic_type tinyint(3) DEFAULT '0' NOT NULL,
            topic_first_post_id mediumint(8) UNSIGNED DEFAULT '0' NOT NULL,
            topic_last_post_id mediumint(8) UNSIGNED DEFAULT '0' NOT NULL,
            topic_moved_id mediumint(8) UNSIGNED DEFAULT '0' NOT NULL
            '''
            # change this for a different forum
            newtopicid=topicmap['%i/%s'%(topicid,board)] # map the topic id so that we get a "real" topic ID
            topicdata[newtopicid]=[newtopicid,topicslist[board][topicid][0],forum,posts[0][1],posts[-1][1],posts[0][2],posts[0][0],title,len(posts)-1,viewcount,topicslist[board][topicid][1]]
            for post in posts:
                '''
              phpbb_posts
                post_id mediumint(8) UNSIGNED NOT NULL auto_increment,
                topic_id mediumint(8) UNSIGNED DEFAULT '0' NOT NULL,
                forum_id smallint(5) UNSIGNED DEFAULT '0' NOT NULL,
                poster_id mediumint(8) DEFAULT '0' NOT NULL,
                post_time int(11) DEFAULT '0' NOT NULL,
                poster_ip char(8) NOT NULL,
                post_username varchar(25),
                enable_bbcode tinyint(1) DEFAULT '1' NOT NULL,
                enable_html tinyint(1) DEFAULT '0' NOT NULL,
                enable_smilies tinyint(1) DEFAULT '1' NOT NULL,
                enable_sig tinyint(1) DEFAULT '1' NOT NULL,
                post_edit_time int(11),
                post_edit_count smallint(5) UNSIGNED DEFAULT '0' NOT NULL
                
              phpbb_posts_text
                post_id mediumint(8) UNSIGNED DEFAULT '0' NOT NULL,
                bbcode_uid char(10) DEFAULT '' NOT NULL,
                post_subject char(60),
                post_text text
                '''
                ''' 0    1  2      3    4        5        6    '''
                '''(time,id,userid,name,edittime,editname,data)'''
                # change this for a different forum
                bbcode_uid='%010x'%random.randint(0,0xFFFFFFFFFFL)
                post[6]=post[6].replace(":<UID>",":"+bbcode_uid).replace('<br>','\n').replace('<br/>','\n')
                postdata+=[[[post[1],newtopicid,forum,post[2],post[0],'00000000',post[3],post[4],bool(post[4])*1],
                             [post[1],bbcode_uid,title,post[6]]]]
            printline(statusline+"Completed: %i post%s"%(len(posts),int(len(posts)!=1)*'s'))
            print # blank line for next topic
        except KeyboardInterrupt: break # abort processing this board
        except Exception:
            do_except('Topic ID %s in Board %s'%(topicid,board), statusline)
# POST ID RENUMBERING
# FORUM STATISTIC COUNTING
forum_stats={}
print "Renumbering Posts"
postdata.sort(key=lambda x:x[0][4]) # sort by date [5th value; index 4]
postid=1
for i,pd in enumerate(postdata): # i will be index, pd will be the data
    oldid=pd[0][0] # save old id
    postdata[i][0][0]=postid # set new posts: post_id
    postdata[i][1][0]=postid # set new posts_text: post_id
    if topicdata[pd[0][1]][3]==oldid: topicdata[pd[0][1]][3]=postid # check ID_FIRST_MSG
    if topicdata[pd[0][1]][4]==oldid: topicdata[pd[0][1]][4]=postid # check ID_LAST_MSG
    if pd[0][2] not in forum_stats: forum_stats[pd[0][2]] = [0,0,0]
    forum_stats[pd[0][2]][1]+=1
    forum_stats[pd[0][2]][2]=postid
    postid+=1
for td in topicdata:
    forum_stats[topicdata[td][2]][0]+=1
# END RENUMBERING
out=open('topics_posts_phpBB.sql','w')
out.write('TRUNCATE TABLE `%stopics`;TRUNCATE TABLE `%sposts`;TRUNCATE TABLE `%sposts_text`;'%(table_prefix,table_prefix,table_prefix))
# change these two for a different forum
topicdata_vals=[escape_vars(i) for i in topicdata.values()]
postdata=[[escape_vars(c) for c in i] for i in postdata]
for i in range(0,len(topicdata_vals),topics_max_group):
    out.write("INSERT INTO `%stopics` (`topic_id`,`topic_type`,`forum_id`,`topic_first_post_id`,`topic_last_post_id`,`topic_poster`,`topic_time`,`topic_title`,`topic_replies`,`topic_views`,`topic_status`) VALUES "%table_prefix)
    out.write(',\n'.join(["(%i,%i,%i,%i,%i,%i,%i,'%s',%i,%i,%i)"%tuple(td) for td in topicdata_vals[i:i+topics_max_group]]))
    out.write(";\n")
for i in range(0,len(postdata),posts_max_group):
    out.write("INSERT INTO `%sposts` (`post_id`,`topic_id`,`forum_id`,`poster_id`,`post_time`,`poster_ip`,`post_username`,`post_edit_time`,`post_edit_count`) VALUES "%table_prefix)
    out.write(',\n'.join(["(%i,%i,%i,%i,%i,'%s','%s',%i,%i)"%tuple(pd[0]) for pd in postdata[i:i+posts_max_group]]))
    out.write(";\n")
for i in range(0,len(postdata),posts_max_group):
    out.write("INSERT INTO `%sposts_text` (`post_id`,`bbcode_uid`,`post_subject`,`post_text`) VALUES "%table_prefix)
    out.write(',\n'.join(["(%i,'%s','%s','%s')"%tuple(pd[1]) for pd in postdata[i:i+posts_max_group]]))
    out.write(";\n")
for fs in forum_stats: out.write('UPDATE `%sforums` SET forum_topics=%i,forum_posts=%i,forum_last_post_id=%i WHERE forum_id=%i;\n'%(table_prefix,forum_stats[fs][0],forum_stats[fs][1],forum_stats[fs][2],fs))
out.close()
print 'Elapsed Conversion Time %.02f seconds' % (time() - beg_time)
print 'Converted',len(postdata),'posts and',len(topicdata),'topics.'
raw_input("Complete! Press <enter> to exit.")
